import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import ddf.minim.spi.*; 
import ddf.minim.signals.*; 
import ddf.minim.*; 
import ddf.minim.analysis.*; 
import ddf.minim.ugens.*; 
import ddf.minim.effects.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class Habitat10 extends PApplet {







Minim minim;
AudioPlayer audioplayer;

public void setup() {
  frameRate(60);
  smooth();
  rectMode(CENTER); // draw from center point, height and width are total (not radii)
  size(800, 600);
  player = new Player();
  inventoryBlocks = new GameObj[player.inventory.length];
  setupInventoryCollision();
  colorMode(HSB, 1);

  minim = new Minim(this);
  if(random(0, 1) < 0.5f){
    audioplayer = minim.loadFile("Tigoolio_-_Oblivion.mp3");
  } else {
    audioplayer = minim.loadFile("Tigoolio_-_Night_lounge.mp3");
  }
  audioplayer.play();
  audioplayer.loop();

  for (int i = -500; i <500; i++) { // this is the y column
    int howManyBlocks = (int)(noise(i*.01f) * 100);
    //println("How many blocks for column " + i + "?: " + howManyBlocks);
    for (int j = 0; j < howManyBlocks; j++) {
      //PVector terrainPos = new PVector(i*terrainSize, ((int)(random(-60, 0))*terrainSize));
      //println("Making terrain at " + (i*terrainSize) + ", " + (-j*terrainSize));
      PVector terrainPos = new PVector(i*terrainSize, -j*terrainSize);
      Terrain t = new Terrain(terrainCount, terrainPos);
      //terrain.add(t);
      insertIntoWorldGrid(t);
      terrainCount++;
    }
  }
}

public void update() {
  for (int i = (int)(player.position.x-450); i < (int)(player.position.x+450); i+= 15) {
    for (int j = (int)(player.position.y-350); j < (int)(player.position.y+350); j+= 15) {
      Terrain t = getTerrainAt(new PVector(i, j));
      if (t != null) t.update();
    }
  }

  player.update();
}

public void draw() {
  update();
  stroke(0,0,0.1f);
  
  // Draw the background
  background(.3f, .1f, .1f);

  // Save this drawing transformation
  pushMatrix();

  // Move our current draw point to the middle of the window
  translate(width/2, height/2);
  // Now, how do you keep the ship on screen?
  // println(myShip.position);
  translate(-player.position.x, -player.position.y);



  for (int i = (int)(player.position.x-450); i < (int)(player.position.x+450); i+= 15) {
    for (int j = (int)(player.position.y-350); j < (int)(player.position.y+350); j+= 15) {
      Terrain t = getTerrainAt(i, j);
      if (t != null) t.draw();
    }
  }

  player.draw();
  
  for(int i = 0; i < circleRoots.size(); i++){
    if(circleRoots.get(i).countChildren() + 1 > 9){
      circleRoots.get(i).drawShineOnChildren();
    }
  }
  
  if(grabbed != null && grabbed.locked){
    fill(0, 1, .8f, .5f);
    ellipse(grabbed.position.x, grabbed.position.y, 15, 15);
  }
  
  popMatrix();
  
  fill(0, 0, 1);
  stroke(0,0,1);
  line(-6200, 1000, -6200, -1000);
  line(6200, 1000, 6200, -1000);
  line(-6200, -1500, 6200, -1000);
  line(-6200, 0, 6200, 0);
  
  //rect(-6200, -1000, 2,2000);
  //rect(6200, -1000, 2, 2000);
  //rect(-6200, -1500, 14000, 2);
  //rect(-6200, 0, 14000, 2);
  
  stroke(0,0,0.1f);
  displayInventoryHUD();
}

public PVector translateToWorld(float x, float y) {
  PVector p = new PVector((x - width/2 + player.position.x), (y - height/2 + player.position.y));
  //println("Translating x " + x + "/y " + y + " to " + p);
  return p;
}

boolean placingBlocks = false;

public void mousePressed() {
  if (mouseY > 80) {
    //println("mouse at: " + mouseX + ", " + mouseY);
    //println("player at: " + player.position.x + ", " + player.position.y);
    //printWorldIndex(translateToWorld(mouseX, mouseY));
    grabbed = getTerrainAt(translateToWorld(mouseX, mouseY));
    if (grabbed != null) {
      //println("grabbed " + grabbed.id + " at " + grabbed.position);
      grabbed.stweight = 1;
    } 
    else {
      placeBlock();
    }
  }
  selectedBoxIndex = checkInventoryCollision();
}

public void mouseDragged() {
    grabbed = getTerrainAt(translateToWorld(mouseX, mouseY));
    if (grabbed != null) {
      //println("grabbed " + grabbed.id + " at " + grabbed.position);
      grabbed.stweight = 1;
    }

}

public void mouseReleased() {
  if (grabbed != null) {
    grabbed.stweight = 0.5f;
    grabbed = null;
  }
}

public void keyPressed() {
  if (key == 'w' || key == ' ') {
    upKey=true;
    player.jump();
  }
  if (key == 'a')
    leftKey=true;
  if (key == 's')
    downKey=true;
  if (key == 'd')
    rightKey=true;

  if (key == CODED) {
    if (keyCode == UP) {
      upKey=true;
      player.jump();
    }
    if (keyCode == LEFT)
      leftKey=true;
    if (keyCode == DOWN)
      downKey=true;
    if (keyCode == RIGHT)
      rightKey=true;
  }
}

public void keyReleased() {
  if (key == 'w' || key == ' ')
    upKey=false;
  if (key == 'a')
    leftKey=false;
  if (key == 's')
    downKey=false;
  if (key == 'd')
    rightKey=false;

  if (key == CODED) {
    if (keyCode == UP)
      upKey=false;
    if (keyCode == LEFT)
      leftKey=false;
    if (keyCode == DOWN)
      downKey=false;
    if (keyCode == RIGHT)
      rightKey=false;
  }
}



Player player;
Terrain grabbed = null;
int terrainCount = 0;
int worldWidth = 1000;
int worldHeight = 500;
int terrainSize = 15;
Terrain[][] worldGrid = new Terrain[worldWidth][worldHeight];
ArrayList<Terrain> circleRoots = new ArrayList<Terrain>();
Boolean upKey = false;
Boolean leftKey = false; 
Boolean downKey = false;
Boolean rightKey = false;

class GameObj {
  PVector position;
  PVector dimensions;
  PVector velocity;
  PVector acceleration;
  PVector colorVals;
  float H, S, B;
  boolean dontdraw;
  
  GameObj(PVector pos, PVector dimensions) {
    this.position = pos;
    this.dimensions = dimensions;
    this.velocity = new PVector(0,0);
    this.acceleration = new PVector(0,0);
    dontdraw = false;
    H = 0.5f;
    S = 0.5f;
    B = 0.5f;
  }
  
  public void update() {
    
  }
  
  public void setColor(float h, float s, float b){
    H = h;
    S = s;
    B = b;
  }
  
  public void draw() {
    // draw a rect
    //noStroke();
    strokeWeight(.5f);
    fill(H,S,B);
    rect(position.x, position.y, dimensions.x, dimensions.y);
  }

}
class Player extends GameObj {

  boolean touchingGround = true;
  PVector lastFrame;
  ArrayList[] inventory;
  //ArrayList<Terrain> inventory;

  Player() {
    super(new PVector(0, -100), new PVector(30, 30));
    inventory = new ArrayList[36];
    for(int i = 0; i < 36; i++){
      inventory[i] = new ArrayList();
    }
    acceleration.x = 0;
    acceleration.y = 0.3f; // gravity
    this.S = 0.8f;
    this.H = 0.73f;
  }

  public void update() {
    lastFrame = new PVector(position.x, position.y);

    if (touchingGround) {
      if (rightKey)velocity.x += .6f;
      if (leftKey)velocity.x += -.6f;
    } 
    else {
      if (rightKey)velocity.x += .06f;
      if (leftKey)velocity.x += -.06f;
    }

    velocity.x += acceleration.x;
    velocity.y += acceleration.y;

    if (touchingGround) velocity.x /= 1.1f;

    position.x += velocity.x;
    position.y += velocity.y;

    checkTerrainCollision();

    if (position.y + (dimensions.y/2) >= 0) { // Replace with terrain check
      position.y = -dimensions.y/2; 
      setRest();
    }
    // bounds checking
    if(position.x < -6200) position.x = -6200;
    if(position.x > 6200) position.x = 6200;
    if(position.y < -1500) position.y = -1500;
  }


  public void jump() {
    if (touchingGround) {
      //println("JUMPING!");
      velocity.y -= 9;
      touchingGround = false;
    }
  }

  public void setRest() {
    touchingGround = true;
    velocity.y = 0;
  }
  
  public void addToInventory(Terrain t){
    int index = (int)(t.H * 36);
    //println("adding terrain with H " + t.H + " to player's inventory at index " + index);
    inventory[index].add(t);
  }

  ArrayList<Terrain> terrainToCheck = new ArrayList<Terrain>();
  public void checkTerrainCollision() {
    boolean collided = false;
    terrainToCheck = new ArrayList<Terrain>();
    for (float i = this.position.x + (terrainSize*2); i >= this.position.x - (terrainSize*3); i -= terrainSize) {
      //println("Player position: " + this.position);
      for (float j = this.position.y + (terrainSize*2); j >= this.position.y - (terrainSize*3); j -= terrainSize) {
        terrainToCheck.add(getTerrainAt(new PVector(i, j)));
      }
    }

    for (Terrain obj : terrainToCheck) {
      if (obj != null) {
        if (grabbed != null && grabbed.id == obj.id && !grabbed.locked) {
          addToInventory(grabbed);
          //println("GRABBED Lineweight: " + grabbed.stweight);
          //println("Attempting to remove grabbed object at position " + grabbed.position);
          //printWorldIndex(grabbed.position);
          removeTerrainAt(new PVector(grabbed.position.x, grabbed.position.y));
          grabUpdateChildren(grabbed);
          grabbed = null;
        } 
        else {

          float verticalDistance = abs(this.position.y - obj.position.y);
          float horizontalDistance = abs(this.position.x - obj.position.x);
          obj.B = 1;
          if ((verticalDistance  < (this.dimensions.y/2) + (obj.dimensions.y/2) ) &&
            (horizontalDistance  < (this.dimensions.x/2) + (obj.dimensions.x/2) )) {
            // WE ARE INSIDE IT, CAPTAIN ! Where shall we go?
            //println("this position: " + this.position);
            //println(verticalDistance + ", " + horizontalDistance + " ||| " + ((this.dimensions.y/2) + (obj.dimensions.y/2)) + ", " + ((this.dimensions.x/2) + (obj.dimensions.x/2)));
            collided = true;
            //obj.S = 0.7;

            float verticalDiscrep = abs(lastFrame.y - obj.position.y) -((this.dimensions.y/2) + (obj.dimensions.y/2)); //USE LAST FRAME
            float horizontalDiscrep = abs(lastFrame.x - obj.position.x) -((this.dimensions.x/2) + (obj.dimensions.x/2));
            if (verticalDiscrep >= horizontalDiscrep) { // fall on top (or bottom)
              //println(obj.position.y + " - " + ((this.dimensions.y/2) + (obj.dimensions.y/2)) + " = " + (obj.position.y - ((this.dimensions.y/2) + (obj.dimensions.y/2))));
              //println("if... " + (this.position.y - (this.dimensions.y/2)) + " <? " + (obj.position.y - (obj.dimensions.y/2)));
              if ((this.position.y - (this.dimensions.y/2)) > (obj.position.y - (obj.dimensions.y/2))) {
                // Bump head? I dunno
                this.position.y = obj.position.y + ((this.dimensions.y/2) + (obj.dimensions.y/2));
                this.velocity.y = 0;
              } 
              else {
                this.position.y = obj.position.y - ((this.dimensions.y/2) + (obj.dimensions.y/2));
                this.setRest();
              }
            } 
            else if (verticalDiscrep != 0) { // collide into wall from the side if we are not already on top of it
              if (this.position.x < obj.position.x) { // to the left
                //println(verticalDiscrep + " >=? " + horizontalDiscrep);
                //println(this.position.x + " <? " + obj.position.x);
                this.position.x = obj.position.x - ((this.dimensions.x/2) + (obj.dimensions.x/2));
                //println("11111 Collided horizontally, setting xVelocity to 0");
                this.velocity.x = 0;
              } 
              else if (this.position.x > obj.position.x) { // to the right
                this.position.x = obj.position.x + ((this.dimensions.x/2) + (obj.dimensions.x/2));
                //println("22222Collided horizontally, setting xVelocity to 0");
                this.velocity.x = 0;
              }
            }
          }
        }
        if (collided) this.B = 0.9f;
        else this.B = 0.7f;
      }
    }
  }
}

class Terrain extends GameObj {
  boolean atRest = false;
  int id;
  float stweight = 0.5f;

  boolean put = false;
  boolean locked = false;
  Terrain parent;
  ArrayList<Terrain> children;

  Terrain(int id) {
    super(new PVector(0, 0), new PVector(15, 15));
    this.id = id;
    generateColor();
    parent = null;
    children = new ArrayList<Terrain>();
  }

  Terrain(int id, PVector pos) {
    super(pos, new PVector(terrainSize, terrainSize));
    this.id = id;
    generateColor();
    parent = null;
    children = new ArrayList<Terrain>();
  }

  public void generateColor() {
    this.H = noise(.005f*position.x, .005f*position.y);
    this.S = 0.8f;
    this.B = 0.4f;
  }
  public void update() {
    super.update();
  }

  public void draw() {
    // draw a rect
    //noStroke();
    strokeWeight(stweight);
    fill(H, S, B);
    rect(position.x, position.y, dimensions.x, dimensions.y);
    /*
    strokeWeight(3);
     if(parent != null) line(position.x, position.y, parent.position.x, parent.position.y);
     strokeWeight(1);
     for(int i = 0; i < children.size(); i++){
     line(position.x, position.y, children.get(i).position.x, children.get(i).position.y);
     }*/
  }
  
  public void drawShineOnChildren(){
    locked = true;
    drawShine();
    for (int i = 0; i < children.size(); i++) {
      children.get(i).drawShineOnChildren();
    } 
  }
  
  public void drawShine(){
    strokeWeight(0);
    fill(H, S-.5f, B, 0.3f);
    ellipse(position.x, position.y, terrainSize+6, terrainSize+6);
    ellipse(position.x, position.y, terrainSize+3, terrainSize+3);
  }

  public void setRest() {
    atRest = true;
    velocity.y = 0;
  }

  public int countChildren() {
    int numTotalChildren = children.size();
    for (int i = 0; i < children.size(); i++) {
      numTotalChildren += children.get(i).countChildren();
    }
    return numTotalChildren;
  }
}


GameObj[] inventoryBlocks;
int selectedBoxIndex = -1;

public void setupInventoryCollision() {
  for (int i = 0; i < player.inventory.length; i++) {
    inventoryBlocks[i] = new GameObj(new PVector(40 + (i*20), 40), new PVector(10, 10));
    inventoryBlocks[i].setColor(((float)i/(float)player.inventory.length), 0.6f, 1);
  }
}

public int checkInventoryCollision() {
  if (mouseY < 80) {
    for (int i = 0; i < inventoryBlocks.length; i++) {
      float horizontalDistance = abs(mouseX - inventoryBlocks[i].position.x);
      if (horizontalDistance  < (inventoryBlocks[i].dimensions.x/2)) {
        float verticalDistance = abs(mouseY - inventoryBlocks[i].position.y);
        if (verticalDistance  < (inventoryBlocks[i].dimensions.y/2) ) {
          println("Collided with inventory box id " + i);
          return i;
        }
      }
    }
    return -1;
  }
  return selectedBoxIndex;
}

public void displayInventoryHUD() {
  // Display a background bit eventually
  fill(.3f, .01f, .01f, 0.5f);
  rect(40 +((35*20)/2), 40, 40 + 35*20, 40);

  fill(0, 0, 1);
  text("Inventory", 30, 20);

  for (int i = 0; i < player.inventory.length; i++) {
    // Draw a rectangle representative of the color, spaced out
    //println("Color: " + i + "/" + player.inventory.length + " = " + ((float)i/(float)player.inventory.length));
    //fill(((float)i/(float)player.inventory.length), 0.6, 0.8);
    //rect(40 + (i*20), 40, 10, 10);
    inventoryBlocks[i].draw();
    //stroke(0, 0, 1);
    fill(0, 0, 1);
    text(player.inventory[i].size(), 35 + (i*20), 52);
    // With a text of how many colors are in that group
  }

  displaySelectedColor();
}

public void displaySelectedColor() {
  // Display a background bit eventually
  fill(.3f, .01f, .01f, 0.5f);
  rect(width-60, 100, 40, 40);

  fill(0, 0, 1);
  text("Selected", width-86, 80);
  
  if(selectedBoxIndex >= 0){
    fill(inventoryBlocks[selectedBoxIndex].H, inventoryBlocks[selectedBoxIndex].S, inventoryBlocks[selectedBoxIndex].B);
    rect(width-60, 95, 20, 20);
    fill(0, 0, 1);
    text(player.inventory[selectedBoxIndex].size(), width-66, 118);
  } else {
    text("None", width-76, 105);
  }
}

public void insertIntoWorldGrid(Terrain t) {
  //t.position.x = squareToGrid(t.position.x);
  //t.position.y = squareToGrid(t.position.y_)
  int columnIndex = (int)(((t.position.x - (t.dimensions.x/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((t.position.y - (t.dimensions.y/2))/terrainSize) + (worldHeight/2));
  if (columnIndex >= worldWidth || rowIndex >= worldHeight || columnIndex < 0 || rowIndex < 0) {
    println("***** ERROR: INSERT TERRAIN out of bounds");
  } 
  else {
    worldGrid[columnIndex][rowIndex] = t;
  }
}

public Terrain getTerrainAt(PVector p) {
  //p.x = squareToGrid(p.x);
  //p.y = squareToGrid(p.y);
  int columnIndex = (int)(((p.x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((p.y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  if (columnIndex >= worldWidth || rowIndex >= worldHeight || columnIndex < 0 || rowIndex < 0) {
    println("***** ERROR: GET TERRAIN out of bounds");
    return null;
  }
  if (worldGrid[columnIndex][rowIndex] == null) return null;
  else return worldGrid[columnIndex][rowIndex];
}

public Terrain getTerrainAt(int x, int y) {
  //x = squareToGrid(x);
  //y = squareToGrid(y);
  int columnIndex = (int)(((x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  if (columnIndex >= worldWidth || rowIndex >= worldHeight || columnIndex < 0 || rowIndex < 0) {
    println("***** ERROR: GET TERRAIN out of bounds");
    return null;
  }
  if (worldGrid[columnIndex][rowIndex] == null) return null;
  else return worldGrid[columnIndex][rowIndex];
}

public void removeTerrainAt(PVector p) {
  //p.x = squareToGrid(p.x);
  //p.y = squareToGrid(p.y);
  int columnIndex = (int)(((p.x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((p.y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  if (columnIndex >= worldWidth || rowIndex >= worldHeight || columnIndex < 0 || rowIndex < 0) {
    println("***** ERROR: GET TERRAIN out of bounds");
  } 
  else {
    //println("removing Terrain at indexes " + columnIndex + ", " + rowIndex);
    worldGrid[columnIndex][rowIndex] = null;
  }
}

public void printWorldIndex(PVector p) {
  int columnIndex = (int)(((p.x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((p.y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  println("World Index: " + columnIndex + ", " + rowIndex + " for value: " + p);
}

public void printWorldIndex(int x, int y) {
  int columnIndex = (int)(((x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  println("World Index: " + columnIndex + ", " + rowIndex + " for value: " + x + ", " + y);
}

public int squareToGrid(float x) {
  //println("squareing " + x + " to " + (((int)(x/terrainSize))*terrainSize));
  if (x < 15) return (((int)((x-(terrainSize/2))/terrainSize))*terrainSize);
  else return (((int)((x+(terrainSize/2))/terrainSize))*terrainSize);
}

public void placeBlock() {
  // if a box is selected and the player actually has them in their inventory
  if (selectedBoxIndex >= 0 && player.inventory[selectedBoxIndex].size() > 0) {
    PVector positionOfClick = translateToWorld(mouseX, mouseY);
    //println("CLICKED AT translated POSITION " + positionOfClick);
    //printWorldIndex(positionOfClick);
    // Check if we are in range of the player
    //float verticalDistance = abs(player.position.y - positionOfClick.y);
    //float horizontalDistance = abs(player.position.x - positionOfClick.x);
    //if ((verticalDistance  < (terrainSize*4) ) && (horizontalDistance  < (terrainSize*4) )) {

    Terrain t = (Terrain) player.inventory[selectedBoxIndex].get(player.inventory[selectedBoxIndex].size()-1);
    //println("grabbed terrain " + t.id + " from player's inventory index " + selectedBoxIndex);
    t.position = positionOfClick;
    // Square it up to the grid
    t.position.x = squareToGrid(t.position.x);
    t.position.y = squareToGrid(t.position.y);
    //println("position after click " + t.position);
    //printWorldIndex(t.position);
    insertIntoWorldGrid(t);
    putUpdateChildren(t);
    player.inventory[selectedBoxIndex].remove(player.inventory[selectedBoxIndex].size()-1);
    //}
  }
}

public void putUpdateChildren(Terrain t) {
  // reset its tree values
  t.parent = null;
  t.children = new ArrayList<Terrain>();
  t.put = true;
  Terrain adj = null;
  //println("Placed Terrain at " + t.position.x + ", " + t.position.y);
  for (int i = (int)t.position.x - terrainSize*2; i <= t.position.x; i += terrainSize) {
    for (int j = (int)t.position.y - terrainSize*2; j <= t.position.y; j += terrainSize) {
      //println("i,j " + i + ", " + j);
      if (adj == null) {
        adj = getTerrainAt(i, j);
        if (adj != null && adj.id == t.id) { // If you found this terrain, skip it
          //println("Terran is ME! Ignore");
          adj = null;
        }
        if (adj != null && !adj.put) {
          adj = null; // Also ignore pieces that the player has not inserted
        }
      }
    }
  }

  if (adj != null) {
    // We found a previously placed head!
    //println(t.id + " found an adjacent parent " + adj.id);
    t.parent = adj;
    adj.children.add(t);
  } 
  else {
    // This node will now be the parent -- reset its tree values
    t.parent = null;
    t.children = new ArrayList<Terrain>();
  }
  //println("Added node to tree with number of nodes: " + countNodesInTreeOf(t));
  if (countNodesInTreeOf(t) > 9 && !circleRoots.contains(getRootOf(t))) {
    //println("Adding a root node to circleRoots");
    circleRoots.add(getRootOf(t));
  }
}

public void grabUpdateChildren(Terrain t) {
  if (t.parent == null) {
    if (t.children.size() > 0) { // If it has children
      Terrain newParent = t.children.get(0); // Grab the first one, make it the parent
      newParent.parent = null;
      //println("1 Grab update: Parent == null, children != null");
      for (int i = 1; i < t.children.size(); i++) { // Grab all the other children and attach them to the new parent
        Terrain otherChild = t.children.get(i);
        otherChild.parent = newParent;
        newParent.children.add(otherChild);
        //println("1 Adding child " + otherChild.id + " to new parent node " + newParent.id);
      }
      //println("1 New Child-Parent has nodes: " + countNodesInTreeOf(newParent));
    }
  } 
  else {
    // We have a parent, attach all children to that parent
    //println("2Grab update: parent != null");
    /*
    Terrain adj = null;
     // find if any placed tiles are around that are not in this tree
     for (int i = (int)t.position.x - terrainSize*2; i <= t.position.x; i += terrainSize) {
     for (int j = (int)t.position.y - terrainSize*2; j <= t.position.y; j += terrainSize) {
     //println("i,j " + i + ", " + j);
     if (adj == null) {
     adj = getTerrainAt(i, j);
     if (adj != null && adj.id == t.id) { // If you found this terrain, skip it
     //println("Terran is ME! Ignore");
     adj = null;
     }
     if (adj != null && !adj.put) {
     adj = null; // Also ignore pieces that the player has not inserted
     }
     if (adj != null && t.parent.child.contains(adj)){
     adj = null;
     }
     }
     }
     }
     if(adj == null){
     println("2 Did not find a new adj thing, make this the root");
     } else {
     println("2 Found a new adj node to use as a parent");
     for (int i = 0; i < t.children.size(); i++) { // Grab all the other children and attach them to the new parent
     println("Adding child " + t.children.get(i).id + " to this parent's node " + t.parent.id);
     t.parent.children.add(t.children.get(i));
     }
     }*/

    for (int i = 0; i < t.children.size(); i++) { // Grab all the other children and attach them to the new parent
      //println("Adding child " + t.children.get(i).id + " to this parent's node " + t.parent.id);
      t.parent.children.add(t.children.get(i));
    }



    //println("Deleting a child... " + t.parent.countChildren());
    t.parent.children.remove(t);
    //println("Deleted child? " + t.parent.countChildren());
    //println("Parent now has total children: " + countNodesInTreeOf(t.parent));
  }

  // Reset the fields for later use
  t.put = false;
  t.parent = null;
  t.children = new ArrayList<Terrain>();
}

public Terrain getRootOf(Terrain t) {
  Terrain root = t.parent;
  Terrain child = t;
  //if (root == null) println("1 I am the parent! I have " + t.children.size());
  while (root != null) {
    child = root;
    root = root.parent;
  }
  return child;
}

public int countNodesInTreeOf(Terrain t) {
  Terrain root = t.parent;
  Terrain child = t;
  //if (root == null) println("2 I am the parent! I have " + t.children.size());
  while (root != null) {
    child = root;
    root = root.parent;
  }
  // 1 (this one) + recursive call

  return 1 + child.countChildren();
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "Habitat10" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
