import ddf.minim.spi.*;
import ddf.minim.signals.*;
import ddf.minim.*;
import ddf.minim.analysis.*;
import ddf.minim.ugens.*;
import ddf.minim.effects.*;
Minim minim;
AudioPlayer audioplayer;

void setup() {
  frameRate(60);
  smooth();
  rectMode(CENTER); // draw from center point, height and width are total (not radii)
  size(800, 600);
  player = new Player();
  inventoryBlocks = new GameObj[player.inventory.length];
  setupInventoryCollision();
  colorMode(HSB, 1);

  minim = new Minim(this);
  if(random(0, 1) < 0.5){
    audioplayer = minim.loadFile("Tigoolio_-_Oblivion.mp3");
  } else {
    audioplayer = minim.loadFile("Tigoolio_-_Night_lounge.mp3");
  }
  audioplayer.play();
  audioplayer.loop();

  for (int i = -500; i <500; i++) { // this is the y column
    int howManyBlocks = (int)(noise(i*.01) * 100);
    //println("How many blocks for column " + i + "?: " + howManyBlocks);
    for (int j = 0; j < howManyBlocks; j++) {
      //PVector terrainPos = new PVector(i*terrainSize, ((int)(random(-60, 0))*terrainSize));
      //println("Making terrain at " + (i*terrainSize) + ", " + (-j*terrainSize));
      PVector terrainPos = new PVector(i*terrainSize, -j*terrainSize);
      Terrain t = new Terrain(terrainCount, terrainPos);
      //terrain.add(t);
      insertIntoWorldGrid(t);
      terrainCount++;
    }
  }
}

void update() {
  for (int i = (int)(player.position.x-450); i < (int)(player.position.x+450); i+= 15) {
    for (int j = (int)(player.position.y-350); j < (int)(player.position.y+350); j+= 15) {
      Terrain t = getTerrainAt(new PVector(i, j));
      if (t != null) t.update();
    }
  }

  player.update();
}

void draw() {
  update();
  stroke(0,0,0.1);
  
  // Draw the background
  background(.3, .1, .1);

  // Save this drawing transformation
  pushMatrix();

  // Move our current draw point to the middle of the window
  translate(width/2, height/2);
  // Now, how do you keep the ship on screen?
  // println(myShip.position);
  translate(-player.position.x, -player.position.y);



  for (int i = (int)(player.position.x-450); i < (int)(player.position.x+450); i+= 15) {
    for (int j = (int)(player.position.y-350); j < (int)(player.position.y+350); j+= 15) {
      Terrain t = getTerrainAt(i, j);
      if (t != null) t.draw();
    }
  }

  player.draw();
  
  for(int i = 0; i < circleRoots.size(); i++){
    if(circleRoots.get(i).countChildren() + 1 > 9){
      circleRoots.get(i).drawShineOnChildren();
    }
  }
  
  if(grabbed != null && grabbed.locked){
    fill(0, 1, .8, .5);
    ellipse(grabbed.position.x, grabbed.position.y, 15, 15);
  }
  
  popMatrix();
  
  fill(0, 0, 1);
  stroke(0,0,1);
  line(-6200, 1000, -6200, -1000);
  line(6200, 1000, 6200, -1000);
  line(-6200, -1500, 6200, -1000);
  line(-6200, 0, 6200, 0);
  
  //rect(-6200, -1000, 2,2000);
  //rect(6200, -1000, 2, 2000);
  //rect(-6200, -1500, 14000, 2);
  //rect(-6200, 0, 14000, 2);
  
  stroke(0,0,0.1);
  displayInventoryHUD();
}

PVector translateToWorld(float x, float y) {
  PVector p = new PVector((x - width/2 + player.position.x), (y - height/2 + player.position.y));
  //println("Translating x " + x + "/y " + y + " to " + p);
  return p;
}

boolean placingBlocks = false;

void mousePressed() {
  if (mouseY > 80) {
    //println("mouse at: " + mouseX + ", " + mouseY);
    //println("player at: " + player.position.x + ", " + player.position.y);
    //printWorldIndex(translateToWorld(mouseX, mouseY));
    grabbed = getTerrainAt(translateToWorld(mouseX, mouseY));
    if (grabbed != null) {
      //println("grabbed " + grabbed.id + " at " + grabbed.position);
      grabbed.stweight = 1;
    } 
    else {
      placeBlock();
    }
  }
  selectedBoxIndex = checkInventoryCollision();
}

void mouseDragged() {
    grabbed = getTerrainAt(translateToWorld(mouseX, mouseY));
    if (grabbed != null) {
      //println("grabbed " + grabbed.id + " at " + grabbed.position);
      grabbed.stweight = 1;
    }

}

void mouseReleased() {
  if (grabbed != null) {
    grabbed.stweight = 0.5;
    grabbed = null;
  }
}

void keyPressed() {
  if (key == 'w' || key == ' ') {
    upKey=true;
    player.jump();
  }
  if (key == 'a')
    leftKey=true;
  if (key == 's')
    downKey=true;
  if (key == 'd')
    rightKey=true;

  if (key == CODED) {
    if (keyCode == UP) {
      upKey=true;
      player.jump();
    }
    if (keyCode == LEFT)
      leftKey=true;
    if (keyCode == DOWN)
      downKey=true;
    if (keyCode == RIGHT)
      rightKey=true;
  }
}

void keyReleased() {
  if (key == 'w' || key == ' ')
    upKey=false;
  if (key == 'a')
    leftKey=false;
  if (key == 's')
    downKey=false;
  if (key == 'd')
    rightKey=false;

  if (key == CODED) {
    if (keyCode == UP)
      upKey=false;
    if (keyCode == LEFT)
      leftKey=false;
    if (keyCode == DOWN)
      downKey=false;
    if (keyCode == RIGHT)
      rightKey=false;
  }
}



Player player;
Terrain grabbed = null;
int terrainCount = 0;
int worldWidth = 1000;
int worldHeight = 500;
int terrainSize = 15;
Terrain[][] worldGrid = new Terrain[worldWidth][worldHeight];
ArrayList<Terrain> circleRoots = new ArrayList<Terrain>();
Boolean upKey = false;
Boolean leftKey = false; 
Boolean downKey = false;
Boolean rightKey = false;

