void insertIntoWorldGrid(Terrain t) {
  //t.position.x = squareToGrid(t.position.x);
  //t.position.y = squareToGrid(t.position.y_)
  int columnIndex = (int)(((t.position.x - (t.dimensions.x/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((t.position.y - (t.dimensions.y/2))/terrainSize) + (worldHeight/2));
  if (columnIndex >= worldWidth || rowIndex >= worldHeight || columnIndex < 0 || rowIndex < 0) {
    println("***** ERROR: INSERT TERRAIN out of bounds");
  } 
  else {
    worldGrid[columnIndex][rowIndex] = t;
  }
}

Terrain getTerrainAt(PVector p) {
  //p.x = squareToGrid(p.x);
  //p.y = squareToGrid(p.y);
  int columnIndex = (int)(((p.x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((p.y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  if (columnIndex >= worldWidth || rowIndex >= worldHeight || columnIndex < 0 || rowIndex < 0) {
    println("***** ERROR: GET TERRAIN out of bounds");
    return null;
  }
  if (worldGrid[columnIndex][rowIndex] == null) return null;
  else return worldGrid[columnIndex][rowIndex];
}

Terrain getTerrainAt(int x, int y) {
  //x = squareToGrid(x);
  //y = squareToGrid(y);
  int columnIndex = (int)(((x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  if (columnIndex >= worldWidth || rowIndex >= worldHeight || columnIndex < 0 || rowIndex < 0) {
    println("***** ERROR: GET TERRAIN out of bounds");
    return null;
  }
  if (worldGrid[columnIndex][rowIndex] == null) return null;
  else return worldGrid[columnIndex][rowIndex];
}

void removeTerrainAt(PVector p) {
  //p.x = squareToGrid(p.x);
  //p.y = squareToGrid(p.y);
  int columnIndex = (int)(((p.x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((p.y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  if (columnIndex >= worldWidth || rowIndex >= worldHeight || columnIndex < 0 || rowIndex < 0) {
    println("***** ERROR: GET TERRAIN out of bounds");
  } 
  else {
    //println("removing Terrain at indexes " + columnIndex + ", " + rowIndex);
    worldGrid[columnIndex][rowIndex] = null;
  }
}

void printWorldIndex(PVector p) {
  int columnIndex = (int)(((p.x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((p.y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  println("World Index: " + columnIndex + ", " + rowIndex + " for value: " + p);
}

void printWorldIndex(int x, int y) {
  int columnIndex = (int)(((x-(terrainSize/2))/terrainSize) + (worldWidth/2));
  int rowIndex = (int)(((y-(terrainSize/2))/terrainSize) + (worldHeight/2));
  println("World Index: " + columnIndex + ", " + rowIndex + " for value: " + x + ", " + y);
}

int squareToGrid(float x) {
  //println("squareing " + x + " to " + (((int)(x/terrainSize))*terrainSize));
  if (x < 15) return (((int)((x-(terrainSize/2))/terrainSize))*terrainSize);
  else return (((int)((x+(terrainSize/2))/terrainSize))*terrainSize);
}

void placeBlock() {
  // if a box is selected and the player actually has them in their inventory
  if (selectedBoxIndex >= 0 && player.inventory[selectedBoxIndex].size() > 0) {
    PVector positionOfClick = translateToWorld(mouseX, mouseY);
    //println("CLICKED AT translated POSITION " + positionOfClick);
    //printWorldIndex(positionOfClick);
    // Check if we are in range of the player
    //float verticalDistance = abs(player.position.y - positionOfClick.y);
    //float horizontalDistance = abs(player.position.x - positionOfClick.x);
    //if ((verticalDistance  < (terrainSize*4) ) && (horizontalDistance  < (terrainSize*4) )) {

    Terrain t = (Terrain) player.inventory[selectedBoxIndex].get(player.inventory[selectedBoxIndex].size()-1);
    //println("grabbed terrain " + t.id + " from player's inventory index " + selectedBoxIndex);
    t.position = positionOfClick;
    // Square it up to the grid
    t.position.x = squareToGrid(t.position.x);
    t.position.y = squareToGrid(t.position.y);
    //println("position after click " + t.position);
    //printWorldIndex(t.position);
    insertIntoWorldGrid(t);
    putUpdateChildren(t);
    player.inventory[selectedBoxIndex].remove(player.inventory[selectedBoxIndex].size()-1);
    //}
  }
}

void putUpdateChildren(Terrain t) {
  // reset its tree values
  t.parent = null;
  t.children = new ArrayList<Terrain>();
  t.put = true;
  Terrain adj = null;
  //println("Placed Terrain at " + t.position.x + ", " + t.position.y);
  for (int i = (int)t.position.x - terrainSize*2; i <= t.position.x; i += terrainSize) {
    for (int j = (int)t.position.y - terrainSize*2; j <= t.position.y; j += terrainSize) {
      //println("i,j " + i + ", " + j);
      if (adj == null) {
        adj = getTerrainAt(i, j);
        if (adj != null && adj.id == t.id) { // If you found this terrain, skip it
          //println("Terran is ME! Ignore");
          adj = null;
        }
        if (adj != null && !adj.put) {
          adj = null; // Also ignore pieces that the player has not inserted
        }
      }
    }
  }

  if (adj != null) {
    // We found a previously placed head!
    //println(t.id + " found an adjacent parent " + adj.id);
    t.parent = adj;
    adj.children.add(t);
  } 
  else {
    // This node will now be the parent -- reset its tree values
    t.parent = null;
    t.children = new ArrayList<Terrain>();
  }
  //println("Added node to tree with number of nodes: " + countNodesInTreeOf(t));
  if (countNodesInTreeOf(t) > 9 && !circleRoots.contains(getRootOf(t))) {
    //println("Adding a root node to circleRoots");
    circleRoots.add(getRootOf(t));
  }
}

void grabUpdateChildren(Terrain t) {
  if (t.parent == null) {
    if (t.children.size() > 0) { // If it has children
      Terrain newParent = t.children.get(0); // Grab the first one, make it the parent
      newParent.parent = null;
      //println("1 Grab update: Parent == null, children != null");
      for (int i = 1; i < t.children.size(); i++) { // Grab all the other children and attach them to the new parent
        Terrain otherChild = t.children.get(i);
        otherChild.parent = newParent;
        newParent.children.add(otherChild);
        //println("1 Adding child " + otherChild.id + " to new parent node " + newParent.id);
      }
      //println("1 New Child-Parent has nodes: " + countNodesInTreeOf(newParent));
    }
  } 
  else {
    // We have a parent, attach all children to that parent
    //println("2Grab update: parent != null");
    /*
    Terrain adj = null;
     // find if any placed tiles are around that are not in this tree
     for (int i = (int)t.position.x - terrainSize*2; i <= t.position.x; i += terrainSize) {
     for (int j = (int)t.position.y - terrainSize*2; j <= t.position.y; j += terrainSize) {
     //println("i,j " + i + ", " + j);
     if (adj == null) {
     adj = getTerrainAt(i, j);
     if (adj != null && adj.id == t.id) { // If you found this terrain, skip it
     //println("Terran is ME! Ignore");
     adj = null;
     }
     if (adj != null && !adj.put) {
     adj = null; // Also ignore pieces that the player has not inserted
     }
     if (adj != null && t.parent.child.contains(adj)){
     adj = null;
     }
     }
     }
     }
     if(adj == null){
     println("2 Did not find a new adj thing, make this the root");
     } else {
     println("2 Found a new adj node to use as a parent");
     for (int i = 0; i < t.children.size(); i++) { // Grab all the other children and attach them to the new parent
     println("Adding child " + t.children.get(i).id + " to this parent's node " + t.parent.id);
     t.parent.children.add(t.children.get(i));
     }
     }*/

    for (int i = 0; i < t.children.size(); i++) { // Grab all the other children and attach them to the new parent
      //println("Adding child " + t.children.get(i).id + " to this parent's node " + t.parent.id);
      t.parent.children.add(t.children.get(i));
    }



    //println("Deleting a child... " + t.parent.countChildren());
    t.parent.children.remove(t);
    //println("Deleted child? " + t.parent.countChildren());
    //println("Parent now has total children: " + countNodesInTreeOf(t.parent));
  }

  // Reset the fields for later use
  t.put = false;
  t.parent = null;
  t.children = new ArrayList<Terrain>();
}

Terrain getRootOf(Terrain t) {
  Terrain root = t.parent;
  Terrain child = t;
  //if (root == null) println("1 I am the parent! I have " + t.children.size());
  while (root != null) {
    child = root;
    root = root.parent;
  }
  return child;
}

int countNodesInTreeOf(Terrain t) {
  Terrain root = t.parent;
  Terrain child = t;
  //if (root == null) println("2 I am the parent! I have " + t.children.size());
  while (root != null) {
    child = root;
    root = root.parent;
  }
  // 1 (this one) + recursive call

  return 1 + child.countChildren();
}

